# README

PyConCodeSe is a bug localisation tool: 
given a bug report it suggests 10 code files that may contain the bug. 

For PyConCodeSe to work it needs a [Tree-sitter](https://tree-sitter.github.io/tree-sitter/) grammar 
to parse your code (we currently support Java, C#, PHP and Rust) and 
the [spaCy](http://spacy.io) library to parse the bug report and any natural language text in the code.

After installing the software as per the following instructions,
you will need to edit a configuration file for each code repo.
The configuration tells PyConCodeSe where your code and your bug reports are.

## Installation

You need at least Python 3.8 to run PyConCodeSe and we strongly recommend you install it in a 
[virtual environment](https://packaging.python.org/en/latest/guides/installing-using-pip-and-virtual-environments).
On Unix (incl. macOS) you would type in a terminal something like:
```bash
python3.11 -m venv venv
source venv/bin/activate
```
to create and activate a virtual environment in subfolder `venv` of your current folder.
Next, type
```bash
pip install -i py-concodese
```
to install everything in the current environment and create the main executable script `pyconcodese`.
Finally, set up spaCy with
```bash
python -m spacy download en_core_web_sm
```
Now you can run `pyconcodese` from any folder, as long as you have the virtual environment activated.

## Configuration

When running PyConCodeSe, if there's no file `config.toml` in the current folder, 
you're prompted to create one with example values.
```bash
pyconcodese 

The PyConcodese config file 'config.toml' was not found in this path. 
 Do you want to copy an example file here? [y/N]y
Example config.toml file copied in the current directory.  
 Please make sure you change the fields accordingly
```
Open the configuration file and edit the paths of the various folders:
```toml
[py-concodese]
# the path that contains grammars (e.g. the folder that contains "tree-sitter-c-master")
# If you don't have a grammar fplder, Set up an empty folder here and Pyconcodese will download the grammars from Github
grammar_path = "/tmp/Pyconcodese/Grammars"

# the next 3 can be the root directory of the application if you wish (e.g.)
# the directory that you would like to store the sqlite database(s) in
sqlite_path = "/tmp/Pyconcodese"
# the directory that you would like to store the vsm data folders in
vsm_path = "/tmp/Pyconcodese"
# where to store output files generated by the application
output_path = "/tmp/Pyconcodese"

derby_path = "/tmp/Pyconcodese/derby"
```
The `grammar_path` is the folder where the Tree-sitter grammar files will be put.
When running `pyconcodese`, you will be asked if you want to update the grammars:
this will automatically download the latest version from GitHub to the `grammar_path` folder.
You can share the grammar folder between projects, but make sure the `grammar_path` you select 
is an empty folder when running PyConCodeSe for the first time.
The grammar update will fail if the folder already contains files that are not git-versioned
but have the same name as the ones on GitHub. 


```toml
[dataset]
bug_repository_file = "/tmp/Pyconcodese/bug_repository_file.xml"
src_path = "/tmp/Pyconcodese"
```

The config.toml file will be created with the current path prefilled as a starting point for all these values.

The bug repository file can be either json or xml, and it has the following structure:
```json
{
  "closed_issues": {
    "1": {
      "issue_url": "https://github.com/rust-lang/libc/issues/3085",
      "issue_id": "#3085",
      "issue_summary": "Missing PR_SET_PTRACER_ANY",
      "issue_description": "JonathanWoollett-Light commented 2 weeks agoJan 22, 2023 \u2022\nedited\nPR_SET_PTRACER is present (https://docs.rs/libc/latest/libc/constant.PR_SET_PTRACER.html) but PR_SET_PTRACER_ANY is missing (https://docs.rs/libc/latest/libc/constant.PR_SET_PTRACER.html?search=PR_SET_PTRACER_ANY).",
      "issue_status": "Closed",
      "issue_reporting_time": "",
      "fixed_by": "#3086",
      "pull_request_summary": "linux: add PR_SET_PTRACER_ANY",
      "pull_request_description": "Contributor\nhowjmay commented 2 weeks agoJan 23, 2023 \u2022\nedited\nfixed #3085",
      "pull_request_status": "Merged",
      "issue_fixed_time": "",
      "files_changed": [
        [
          "1",
          "libc-test/semver/fuchsia.txt"
        ],
        [
          "1",
          "libc-test/semver/linux.txt"
        ],
        [
          "1",
          "src/fuchsia/mod.rs"
        ],
        [
          "1",
          "src/unix/linux_like/emscripten/mod.rs"
        ],
        [
          "1",
          "src/unix/linux_like/linux/mod.rs"
        ]
      ]
    },
    "4": {
      "issue_url": "https://github.com/rust-lang/libc/issues/3026",
      "issue_id": "#3026",
      "issue_summary": "PIDFD_NONBLOCK missing from Linux armv7 uclibceabihf",
      "issue_description": "JonathanWoollett-Light commented on Dec 5, 2022Dec 5, 2022\nhttps://github.com/nix-rust/nix/pull/1868/checks?check_run_id=9877370516",
      "issue_status": "Closed",
      "issue_reporting_time": "",
      "fixed_by": "#3029",
      "pull_request_summary": "PIDFD_NONBLOCK addition into uclibc.",
      "pull_request_description": "Contributor\ndevnexen commented on Dec 6, 2022Dec 6, 2022\ncloses #3026.",
      "pull_request_status": "Merged",
      "issue_fixed_time": "",
      "files_changed": [
        [
          "1",
          "src/unix/linux_like/linux/uclibc/arm/mod.rs"
        ],
        [
          "1",
          "src/unix/linux_like/linux/uclibc/mips/mod.rs"
        ],
        [
          "1",
          "src/unix/linux_like/linux/uclibc/x86_64/mod.rs"
        ]
      ]
    }
  }
}
```

```xml
<?xml version="1.0" encoding="ISO-8859-1"?>

<bugrepository name="SWT">
  <bug id="88829" opendate="2005-03-22 20:41:00" fixdate="2005-04-06 17:05:00">
    <buginformation>
      <summary>Table.setColumnOrder() may not fire enough Move events</summary>
      <description>- start with five columns, all different widths - do Table.setColumnOrder(new int[] {4,1,2,3,0}); - SWT.Move events are fired for columns 0 and 4 because they swapped positions -&amp;gt; but Move should have been fired for all of the columns since the width of the first displayed column changed, and therefore all of the other columns are auto-shifted accordingly</description>
    </buginformation>
    <fixedFiles>
      <file>org.eclipse.swt.widgets.Table.java</file>
    </fixedFiles>
  </bug>
  <bug id="90018" opendate="2005-04-01 14:40:00" fixdate="2005-04-05 08:14:00">
    <buginformation>
      <summary>Native tooltips left around on CTabFolder</summary>
      <description>Hover over the PartStack CTabFolder inside eclipse until some native tooltip is displayed. For example, the maximize button. When the tooltip appears, change perspectives using the keybinding. the CTabFolder gets hidden, but its tooltip is permanently displayed and never goes away. Even if that CTabFolder is disposed (I'm assuming) when the perspective is closed.</description>
    </buginformation>
    <fixedFiles>
      <file>org.eclipse.swt.custom.CTabFolder.java</file>
    </fixedFiles>
  </bug>
</bugrepository>
```

